package com.docus.server.common.netty.server;

import com.docus.server.enums.OnlineStateEnum;
import com.docus.server.service.ISchTerminatorService;
import io.netty.channel.Channel;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 客户端IP和通信信道的映射
 */
@Component
@Slf4j
public class ChannelRepository {

    @Resource
    private ISchTerminatorService iSchTerminatorService;

    /**
     * <IP-Channel>
     */
    private final static Map<String, Channel> CHANNEL_CACHE_MAP = new ConcurrentHashMap<>();

    /**
     * 终端上线
     */
    public void put(String key, Channel value, OnlineStateEnum onlineState) {
        //客户端上线
        CHANNEL_CACHE_MAP.put(key, value);
        AttributeKey<String> attributeKey = AttributeKey.valueOf("user");
        value.attr(attributeKey).set(key);

        iSchTerminatorService.saveOrUpdate(key, onlineState);
    }

    public String getClientKey(Channel channel) {
        AttributeKey<String> key = AttributeKey.valueOf("user");
        if (channel.hasAttr(key)) {
            return channel.attr(key).get();
        }
        return null;
    }

    public Channel get(String key) {
        return CHANNEL_CACHE_MAP.get(key);
    }

    public Map<String, Channel> getAll() {
        return CHANNEL_CACHE_MAP;
    }

    /**
     * 终端离线
     */
    public void remove(String key) {
        CHANNEL_CACHE_MAP.remove(key);

        iSchTerminatorService.saveOrUpdate(key, OnlineStateEnum.OFFLINE);
    }

}
