package com.docus.server.service.impl;

import com.docus.core.util.ListUtils;
import com.docus.core.util.property.Setters;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.SchCollectRecordConvert;
import com.docus.server.dto.scheduling.management.schcollectrecord.AddSchCollectRecordDTO;
import com.docus.server.dto.scheduling.management.schcollectrecord.DeleteSchCollectRecordDTO;
import com.docus.server.dto.scheduling.management.schcollectrecord.EditSchCollectRecordDTO;
import com.docus.server.entity.scheduling.management.SchCollectRecord;
import com.docus.server.entity.scheduling.management.SchSystemParams;
import com.docus.server.infrastructure.dao.ISchCollectRecordDao;
import com.docus.server.infrastructure.dao.ISchSystemParamsDao;
import com.docus.server.service.ISchCollectRecordService;
import com.docus.server.vo.scheduling.management.schcollectrecord.SchCollectRecordVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 采集记录表 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-07-15
 */
@Service
public class SchCollectRecordServiceImpl implements ISchCollectRecordService {
    @Resource
    private ISchCollectRecordDao iSchCollectRecordDao;
    @Resource
    private ISchSystemParamsDao iSchSystemParamsDao;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public SchCollectRecordVO findById(String id) {
        return SchCollectRecordConvert.INSTANCE.convertVO(iSchCollectRecordDao.findById(id));
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<SchCollectRecordVO> search(SearchDTO searchDTO) {
        PageResult<SchCollectRecordVO> result = SchCollectRecordConvert.INSTANCE.convertVO(iSchCollectRecordDao.search(searchDTO));

        if (CollectionUtils.isEmpty(result.getList())) {
            return new PageResult<>();
        }

        List<SchSystemParams> schSystemParams = iSchSystemParamsDao.findBy("paramValue", ListUtils.distinctSelect(result.getList(), SchCollectRecordVO::getCollectorId));
        Map<String, SchSystemParams> taskConfigRetryLogMap = ListUtils.toMap(schSystemParams, SchSystemParams::getParamValue);

        Setters.<SchCollectRecordVO>instance().list(result.getList()).cycleSetProperties(p -> {
            String collectorId = String.valueOf(p.getCollectorId());
            if (taskConfigRetryLogMap.containsKey(collectorId)) {
                p.setCollectorName(taskConfigRetryLogMap.get(collectorId).getParamName());
            }
        });

        return result;
    }

    /**
     * 新增
     *
     * @param addSchCollectRecordDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(AddSchCollectRecordDTO addSchCollectRecordDTO) {
        SchCollectRecord schCollectRecord = SchCollectRecordConvert.INSTANCE.convertDO(addSchCollectRecordDTO);
        return iSchCollectRecordDao.add(schCollectRecord);
    }

    /**
     * 编辑
     *
     * @param editSchCollectRecordDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(EditSchCollectRecordDTO editSchCollectRecordDTO) {
        SchCollectRecord schCollectRecord = SchCollectRecordConvert.INSTANCE.convertDO(editSchCollectRecordDTO);
        return iSchCollectRecordDao.edit(schCollectRecord);
    }

    /**
     * 批量删除
     *
     * @param deleteSchCollectRecordDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteSchCollectRecordDTO deleteSchCollectRecordDTO) {
        return iSchCollectRecordDao.delete(deleteSchCollectRecordDTO.getIds());
    }
}

