package com.docus.server.service.impl;

import com.docus.core.util.Func;
import com.docus.core.util.ListUtils;
import com.docus.core.util.property.Setters;
import com.docus.infrastructure.redis.service.IdService;
import com.docus.infrastructure.web.exception.ApiException;
import com.docus.infrastructure.web.exception.ExceptionCode;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.SchCollectorConvert;
import com.docus.server.dto.scheduling.management.schcollector.AddSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.DeleteSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.EditSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.UpdateSchCollectorDTO;
import com.docus.server.entity.scheduling.management.SchCollector;
import com.docus.server.entity.scheduling.management.SchSystemParams;
import com.docus.server.infrastructure.dao.ISchCollectorDao;
import com.docus.server.service.ISchCollectorService;
import com.docus.server.service.ISchSystemParamsService;
import com.docus.server.vo.scheduling.management.schcollector.SchCollectorVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;

/**
 * 采集器管理 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-07-15
 */
@Service
public class SchCollectorServiceImpl implements ISchCollectorService {
    @Resource
    private ISchCollectorDao iSchCollectorDao;
    @Resource
    private ISchSystemParamsService iSchSystemParamsService;
    @Resource
    private IdService idService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public SchCollectorVO findById(String id) {
        return SchCollectorConvert.INSTANCE.convertVO(iSchCollectorDao.findById(id));
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<SchCollectorVO> search(SearchDTO searchDTO) {
        PageResult<SchCollectorVO> result = SchCollectorConvert.INSTANCE.convertVO(iSchCollectorDao.search(searchDTO));

        if (CollectionUtils.isEmpty(result.getList())) {
            return new PageResult<>();
        }

        Map<String, SchSystemParams> map = iSchSystemParamsService.find(ListUtils.distinctSelect(result.getList(), SchCollectorVO::getCollectorId));

        Setters.<SchCollectorVO>instance().list(result.getList()).cycleSetProperties(p -> {
            String collectorId = String.valueOf(p.getCollectorId());
            if (map.containsKey(collectorId)) {
                p.setCollectorName(map.get(collectorId).getParamName());
            }
        });

        return result;
    }

    /**
     * 选为当前版本并更新
     *
     * @return 成功或失败
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateVersion(UpdateSchCollectorDTO updateDTO) {
        SchCollector schCollector = iSchCollectorDao.findOneBy("collectorId", updateDTO.getCollectorId());
        schCollector.setCollectorVersionId(updateDTO.getCollectorVersionId());
        return iSchCollectorDao.updateById(schCollector);
    }

    /**
     * 新增
     *
     * @param addSchCollectorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(AddSchCollectorDTO addSchCollectorDTO) {

        Long collectorId = addSchCollectorDTO.getCollectorId();

        SchCollector schCollectorDO = iSchCollectorDao.findOneBy("collectorId", collectorId);

        if (Func.notNull(schCollectorDO)) {
            throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "采集器类型已经存在");
        }

        SchCollector schCollector = SchCollectorConvert.INSTANCE.convertDO(addSchCollectorDTO);
        schCollector.setId(idService.getDateSeq());
        return iSchCollectorDao.add(schCollector);
    }

    /**
     * 编辑
     *
     * @param editSchCollectorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(EditSchCollectorDTO editSchCollectorDTO) {

        Long id = editSchCollectorDTO.getId();

        SchCollector schCollector = iSchCollectorDao.findById(id);

        if (Func.isNull(schCollector)) {
            throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "采集器不存在，无法更新");
        }

        schCollector.setCollectLanguage(editSchCollectorDTO.getCollectLanguage());
        schCollector.setStartMode(editSchCollectorDTO.getStartMode());
        schCollector.setTaskTimeout(editSchCollectorDTO.getTaskTimeout());
        schCollector.setUpdateTime(new Date());
        return iSchCollectorDao.edit(schCollector);
    }

    /**
     * 批量删除
     *
     * @param deleteSchCollectorDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteSchCollectorDTO deleteSchCollectorDTO) {
        return iSchCollectorDao.delete(deleteSchCollectorDTO.getIds());
    }
}

