package com.docus.server.service.impl;

import com.docus.core.util.Func;
import com.docus.infrastructure.redis.service.IdService;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.SchTerminatorConvert;
import com.docus.server.dto.scheduling.management.schterminator.AddSchTerminatorDTO;
import com.docus.server.dto.scheduling.management.schterminator.DeleteSchTerminatorDTO;
import com.docus.server.dto.scheduling.management.schterminator.EditSchTerminatorDTO;
import com.docus.server.entity.scheduling.management.SchTerminator;
import com.docus.server.enums.BusyStateEnum;
import com.docus.server.enums.OnlineStateEnum;
import com.docus.server.infrastructure.dao.ISchTerminatorDao;
import com.docus.server.service.ISchTerminatorService;
import com.docus.server.vo.scheduling.management.schterminator.SchTerminatorVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 执行管理器 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-07-15
 */
@Service
public class SchTerminatorServiceImpl implements ISchTerminatorService {
    @Resource
    private ISchTerminatorDao iSchTerminatorDao;
    @Resource
    private IdService idService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public SchTerminatorVO findById(String id) {
        return SchTerminatorConvert.INSTANCE.convertVO(iSchTerminatorDao.findById(id));
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<SchTerminatorVO> search(SearchDTO searchDTO) {
        return SchTerminatorConvert.INSTANCE.convertVO(iSchTerminatorDao.search(searchDTO));
    }

    @Override
    public void saveOrUpdate(String terminatorIp, OnlineStateEnum onlineStateEnum) {

        SchTerminator schTerminator = iSchTerminatorDao.findOneBy("terminatorIp", terminatorIp);

        if (Func.isNull(schTerminator)) {
            SchTerminator schTerminatorVO = new SchTerminator();

            schTerminatorVO.setId(idService.getDateSeq());
            schTerminatorVO.setTerminatorIp(terminatorIp);
            schTerminatorVO.setTerminatorName(terminatorIp);
            schTerminatorVO.setOnlineState(onlineStateEnum);
            schTerminatorVO.setBusyState(BusyStateEnum.IDLE);

            iSchTerminatorDao.save(schTerminatorVO);
        } else {

            schTerminator.setBusyState(BusyStateEnum.IDLE);
            schTerminator.setOnlineState(onlineStateEnum);
            schTerminator.setUpdateTime(new Date());

            iSchTerminatorDao.updateById(schTerminator);
        }
    }

    /**
     * 新增
     *
     * @param addSchTerminatorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(AddSchTerminatorDTO addSchTerminatorDTO) {
        SchTerminator schTerminator = SchTerminatorConvert.INSTANCE.convertDO(addSchTerminatorDTO);
        return iSchTerminatorDao.add(schTerminator);
    }

    /**
     * 编辑
     *
     * @param editSchTerminatorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(EditSchTerminatorDTO editSchTerminatorDTO) {
        SchTerminator schTerminator = SchTerminatorConvert.INSTANCE.convertDO(editSchTerminatorDTO);
        return iSchTerminatorDao.edit(schTerminator);
    }

    /**
     * 批量删除
     *
     * @param deleteSchTerminatorDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteSchTerminatorDTO deleteSchTerminatorDTO) {
        return iSchTerminatorDao.delete(deleteSchTerminatorDTO.getIds());
    }
}

